#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <iostream>
#include <vector>

using namespace std;

#include "cv.h"
#include "cvaux.h"
#include "highgui.h"
#include "ml.h"

#include "VideoSource.h"
#include "ArmExtractor.h"

#include "Recognizer.h"
#include "BgSubstitution.h"

#include "HandRecognition.h"

#define WINDOW_ARM1	( 0 )
#define WINDOW_ARM2	( 0 )
#define WINDOW_HAND1	( 1 )
#define WINDOW_HAND2	( 1 )

//#define DEBUG( str )  cout << str << endl;
#define DEBUG( str ) ;

#define STONE ( 0 )
#define PAPER ( 1 )
#define SCISS ( 2 )



class Segmentation {
	public:
		IplImage * bg, *frame;
		IplImage * luvFrame;
		
		CvNormalBayesClassifier * classifier;
		BgSubstitution * bgsub;
		const char * skinFileName;
		CvMat * sample;
		CvMat * classes;
		
	public:
		Segmentation( IplImage * bg, const char * skinFileName ){
			this->bg =  cvCreateImage( cvSize( bg->width, bg->height ), IPL_DEPTH_8U, 3 );
			this->bg->origin = bg->origin;
			cvCopy( bg, this->bg );
			this->frame =  cvCreateImage( cvSize( bg->width, bg->height ), IPL_DEPTH_8U, 3 );
			this->frame->origin = bg->origin;

			classifier = new CvNormalBayesClassifier( );
			this->skinFileName = skinFileName;
			train( bg );
			
			bgsub = new BgSubstitution( bg );
			luvFrame =  cvCreateImage( cvSize( bg->width, bg->height ), IPL_DEPTH_8U, 3 );
			
			
			sample = cvCreateMat( frame->width*frame->height, 2, CV_32FC1 );
			classes = cvCreateMat( frame->width*frame->height, 1, CV_32FC1 );

			
			//cvNamedWindow( "bgsub" );
		}
		~Segmentation( ) {
			if ( bg )cvReleaseImage( &bg );
			if ( luvFrame )cvReleaseImage( &luvFrame );
			
			//cvDestroyWindow( "bgsub" );
			cvReleaseMat( &sample );
			cvReleaseMat( &classes );
		}
		
		void segmentate( IplImage * src, IplImage * dst ) {
			cvCvtColor( src, luvFrame, CV_RGB2Luv );
			
			//bgsub->substitute( luvFrame );
			classify( dst );
			//cvShowImage( "bgsub", dst );
			
		//	cleaning( dst );
		}
		
		void cleaning( IplImage * img ) {
				IplImage * temp = cvCreateImage( cvSize( img->width, img->height ), IPL_DEPTH_8U, 1 );
				
				int iter = 1;
				
				cvErode( img, temp, NULL, iter );
				cvDilate( temp, img, NULL, iter );
				
				cvReleaseImage( &temp );
				
		}
		
		void train( IplImage * bg ) {
			IplImage * skin = cvLoadImage( skinFileName );

			IplImage *luvSkin = cvCreateImage( cvSize( skin->width, skin->height ), IPL_DEPTH_8U, 3 );
			IplImage *luvBg = cvCreateImage( cvSize( bg->width, bg->height ), IPL_DEPTH_8U, 3 );

			cvCvtColor( bg, luvBg, CV_RGB2Luv );
			cvCvtColor( skin, luvSkin, CV_RGB2Luv );
			
			int skinSize =	skin->width * skin->height;
			int bgSize =	bg->width * bg->height;
			
			int trainSize = skinSize + bgSize;
			
			
			CvMat * trainData = cvCreateMat( trainSize, 2, CV_32FC1 );
			CvMat * trainLabels = cvCreateMat( trainSize, 1, CV_32FC1 );
			
			for( int i = 0; i < luvSkin->height; i++ )
				for( int j = 0; j < luvSkin->width; j++ ) {
					float u = luvSkin->imageData[ i*luvSkin->widthStep + j * 3 + 1];
					float v = luvSkin->imageData[ i*luvSkin->widthStep + j * 3 + 2];
					
					cvmSet( trainData, i*luvSkin->width+j, 0, u );
					cvmSet( trainData, i*luvSkin->width+j, 1, v );
					cvmSet( trainLabels, i*luvSkin->width+j, 0, 0.0 );
				}
			
			for( int i = 0; i < luvBg->height; i++ )
				for( int j = 0; j < luvBg->width; j++ ) {
					
					float u = luvBg->imageData[ i*luvBg->widthStep + j * 3 + 1];
					float v = luvBg->imageData[ i*luvBg->widthStep + j * 3 + 2];

					
					cvmSet( trainData, skinSize + i*luvBg->width+j, 0, u );
					cvmSet( trainData, skinSize + i*luvBg->width+j, 1, v );
					cvmSet( trainLabels, skinSize + i*luvBg->width+j, 0, 1 );
				}
				
			classifier->train( trainData, trainLabels );
			
			
			cvReleaseImage( &luvBg );
			cvReleaseImage( &luvSkin );
			
			cvReleaseMat( &trainData );
			cvReleaseMat( &trainLabels );
			
		}


		void classify( IplImage * result ) {
			int w = luvFrame->width, h = luvFrame->height;
			int i, j;
			float u, v, c;
			int offset;
			
			

			for( i = 0; i < h; i++ )
				for( j = 0; j < w; j++ ) {
					u = luvFrame->imageData[ i*luvFrame->widthStep + j * 3 + 1];
					v = luvFrame->imageData[ i*luvFrame->widthStep + j * 3 + 2];

					cvmSet( sample, i*frame->width+j, 0, u );
					cvmSet( sample, i*frame->width+j, 1, v );

/*					cvmSet( sample, 0, 0, u );
					cvmSet( sample, 0, 1, v );

					c = classifier->predict( sample );

					offset = i*result->widthStep + j;
					result->imageData[ offset ] = (c==0)? 255: 0;*/
				}

			classifier->predict( sample, classes );

			for( int i = 0; i < h; i++ ) {
				for( int j = 0; j < w; j++ ) {
					int offset = i*result->widthStep + j;
					if( cvmGet( classes, i*w+j, 0 ) == 0.0 ) {
						result->imageData[ offset ] = 255;
					} else {
						result->imageData[ offset ] = 0;
					}
				}
			}

			result->origin = frame->origin;
		}
};


class Game {
	private:
		IplImage * frame;
		IplImage * seg;
		IplImage * arm1, *arm2, *hand1, *hand2;
		
		int playerNum;
		
		IplImage * sign[6];
		
		vector<int> posList[2];
		
		VideoSource * capture;
		Segmentation * segmentation;

		Recognizer recognizer;
		
		HandRecognition * hr1, *hr2;
		
		int frameArea;
		bool started;
		int direction;
		int cdnum;
		float alphastate;
		
		IplImage * robot[3];
		IplImage * icons[3];
	public:
	
		Game( VideoSource * capture, int playerNum ) {
			this->capture = capture;
			this->playerNum = playerNum;
			
			hr1 = new HandRecognition( 3 );
			hr2 = new HandRecognition( 3 );
			
			 srand((unsigned)time(0));
		}
		~Game( ) {
			delete hr1;
			delete hr2;
			destroy( );
		}
		
		void drawSign( IplImage * img, int N, float alpha ) {
			int midx = frame->height / 2;
			int midy = frame->width / 2;
			int frameOffset, signOffset;
			int i2, j2;
			unsigned char r1, r2, g1, g2, b1, b2;
			unsigned char r, g, b;
			
			IplImage * sign = this->sign[N];
			
			for( int i = 0; i < sign->height; i++ ) {
				for( int j = 0; j < sign->width; j++ ) {
					
					signOffset = (sign->height - i-1) * sign->widthStep + j * 3;
					r2 = (unsigned char)sign->imageData[  signOffset + 0 ];
					g2 = (unsigned char)sign->imageData[  signOffset + 1 ];
					b2 = (unsigned char)sign->imageData[  signOffset + 2 ];
					
					// Black is transparent
					if ( r2 == 0 && g2 == 0 && b2 == 0 ) continue;

					i2 = frame->height/2 + i - sign->height/2;
					j2 = frame->width/2 + j - sign->width/2;
					frameOffset = i2 * frame->widthStep + j2 * 3;
					r1 = (unsigned char)frame->imageData[ frameOffset + 0 ];
					g1 = (unsigned char)frame->imageData[ frameOffset + 1 ];
					b1 = (unsigned char)frame->imageData[ frameOffset + 2 ];
					
					
					r = (unsigned char)(((float)r1)*(1.0f-alpha) + ((float)r2)*alpha);
					g = (unsigned char)(((float)g1)*(1.0f-alpha) + ((float)g2)*alpha);
					b = (unsigned char)(((float)b1)*(1.0f-alpha) + ((float)b2)*alpha);
					
					frame->imageData[ frameOffset + 0 ] = r;
					frame->imageData[ frameOffset + 1 ] = g;
					frame->imageData[ frameOffset + 2 ] = b;
				}
			}
			
		}
		
		void drawIcon( IplImage * img, int N, int side ) {
			
			IplImage * icon = icons[N];
			unsigned char r1, g1, b1;
			int x, y;
			
			for( int i = 0; i < icon->height; i++ ) {
				for( int j = 0; j < icon->width; j++ ) {
					r1 = (unsigned char)icon->imageData[ i * icon->widthStep + j*3 + 0 ];
					g1 = (unsigned char)icon->imageData[ i * icon->widthStep + j*3 + 0 ];
					b1 = (unsigned char)icon->imageData[ i * icon->widthStep + j*3 + 0 ];
					
					
					x = i;
					y = (side == 0)?j: img->width-j-1;
					
					if ( r1 == 0 && g1 == 0 && b1 == 0 ) {
						img->imageData[ x * img->widthStep + y*3 + 0 ] = (unsigned char)((float)(unsigned char)img->imageData[ x * img->widthStep + y*3 + 0 ] * 0.6f);
						img->imageData[ x * img->widthStep + y*3 + 1 ] = (unsigned char)((float)(unsigned char)img->imageData[ x * img->widthStep + y*3 + 1 ] * 0.6f);
						img->imageData[ x * img->widthStep + y*3 + 2 ] = (unsigned char)((float)(unsigned char)img->imageData[ x * img->widthStep + y*3 + 2 ] * 0.6f);
					} else {
						img->imageData[ x * img->widthStep + y*3 + 0 ] = r1;
						img->imageData[ x * img->widthStep + y*3 + 1 ] = g1;
						img->imageData[ x * img->widthStep + y*3 + 2 ] = b1;
					}
					
				}
			}
		}
		
		int calcArea( IplImage * img ) {
			int a = 0;
			
			for( int i = 0; i < img->height; i++ )
				for( int j = 0; j < img->width; j++ )
					if ( (unsigned char)img->imageData[ i*img->widthStep + j ] > 0 )
						a++;
			return a;
		}
		
		void storePositions( void ) {
			int avg1=0, avg2=0;
			int c1=0, c2 = 0;
			
			DEBUG( "st1" );
			
			for( int i = 0; i < frame->height; i++ )
				for( int j = 0; j < frame->width; j++ ) {
					if ( (unsigned char)arm1->imageData[ i*hand1->widthStep + j ] > 0 ) {
						avg1 += i;
						c1++;
					}
					if ( playerNum == 2 ) {
						if ( (unsigned char)arm2->imageData[ i*hand2->widthStep + j ] > 0 ) {
							avg2 += i;
							c2++;
						}
					}
				}

			DEBUG( "st2" );
			if ( c1 > 0 ) avg1 /= c1;
			DEBUG( "st2a" );
			if ( posList[0].size( ) > 12 ) posList[0].erase( posList[0].begin() );
			posList[0].push_back( avg1 );
			DEBUG( "st3" );

			if ( playerNum == 2 ) {
				if ( c2 > 0 ) avg2 /= c2;
				if ( posList[1].size( ) > 12 ) posList[1].erase( posList[1].begin() );
				posList[1].push_back( avg2 );
			}
			DEBUG( "st4" );
			
			//cout << c1 << " " << c2 << "             " << avg1 << " " << avg2 << endl;
		}
		
		int getMovement( void ) {
			int S = posList[0].size( );
			if ( S < 5 ) return 0;
			
			IplImage * arm = arm1;
			if ( (float)calcArea( arm ) / frameArea < 0.02 )
				return 0;
			
			int m[3];
			
			m[0] = posList[0][S-1] - posList[0][S-2];
			m[1] = posList[0][S-2] - posList[0][S-3];
			m[2] = posList[0][S-3] - posList[0][S-4];
			
			
			if ( m[0] <= 0 && m[1] <= 0 && m[2] <= 0 && m[0]+m[1]+m[2]<-20 ) return -1; 
			if ( m[0] >= 0 && m[1] >= 0 && m[2] >= 0 && m[0]+m[1]+m[2]>20  ) return 1; 
			
			return 0;
		}
		
		bool detectStarted( void ) {
			int area1 = calcArea( arm1 );
			float d1 = (float)area1 / frameArea;
			float p1 = ((float)posList[0].back( )) / (float)(frame->height);
			
			float d2, p2;
			
			if ( playerNum == 2 ) {
				int area2 = calcArea( arm2 );
				d2 = (float)area2 / frameArea;
				p2 = ((float)posList[1].back( )) / (float)(frame->height);
			}
			
			
			
			
			//cout << d1 << " " << d2 << "      " << p1 << " " << p2 << endl;
			
			if ( playerNum == 2 ) {
				if ( p1 > 0.2 && p2 > 0.2 && d1 > 0.02 && d2 > 0.02 )
					return true;
			} else {
				if ( p1 > 0.2 && d1 > 0.02 )
					return true;
			}
				
			return false;
		}
		
		void init( void ) {
			direction = 1;
			started = false;
			cdnum = -1;
			alphastate = -1;
			
			IplImage * dummy;
			
			for( int i = 0; i < 5; i++ )
				dummy = capture->getFrame( );
			segmentation = new Segmentation( dummy, "skin.png" );
			
			arm1 = cvCreateImage( cvSize( dummy->width, dummy->height ), IPL_DEPTH_8U, 1 );
			if ( playerNum == 2 )
				arm2 = cvCreateImage( cvSize( dummy->width, dummy->height ), IPL_DEPTH_8U, 1 );
			
			seg = cvCreateImage( cvSize( dummy->width, dummy->height ), IPL_DEPTH_8U, 1 );
			hand1 = cvCreateImage( cvSize( dummy->width, dummy->height ), IPL_DEPTH_8U, 1 );
			hand2 = cvCreateImage( cvSize( dummy->width, dummy->height ), IPL_DEPTH_8U, 1 );

			cvNamedWindow( "Game", CV_WINDOW_AUTOSIZE );
			cvNamedWindow( "Seg", CV_WINDOW_AUTOSIZE );
			if ( WINDOW_ARM1 ) cvNamedWindow( "Arm1", CV_WINDOW_AUTOSIZE );
			if ( WINDOW_HAND1 ) cvNamedWindow( "Hand1", CV_WINDOW_AUTOSIZE );
			if ( playerNum == 2 ) {
				if ( WINDOW_ARM2 ) cvNamedWindow( "Arm2", CV_WINDOW_AUTOSIZE );
				if ( WINDOW_HAND2 ) cvNamedWindow( "Hand2", CV_WINDOW_AUTOSIZE );
			}
			
			frameArea = dummy->width * dummy->height;
			
			posList[0].clear( );
			posList[1].clear( );
			
			sign[0] = cvLoadImage( "resources/3.png" ); sign[0]->origin = dummy->origin;
			sign[1] = cvLoadImage( "resources/2.png" ); sign[1]->origin = dummy->origin;
			sign[2] = cvLoadImage( "resources/1.png" ); sign[2]->origin = dummy->origin;
			sign[3] = cvLoadImage( "resources/go.png" ); sign[3]->origin = dummy->origin;
			sign[4] = cvLoadImage( "resources/p1wins.png" ); sign[4]->origin = dummy->origin;
			sign[5] = cvLoadImage( "resources/p2wins.png" ); sign[5]->origin = dummy->origin;
			sign[6] = cvLoadImage( "resources/compwins.png" ); sign[6]->origin = dummy->origin;
			sign[7] = cvLoadImage( "resources/humwins.png" ); sign[7]->origin = dummy->origin;
			sign[8] = cvLoadImage( "resources/draw.png" ); sign[8]->origin = dummy->origin;

			IplImage * 
			temp = cvLoadImage( "resources/robot1.png" );
			robot[0] = cvCreateImage( cvSize( 300, 300 ), IPL_DEPTH_8U, 3 );
			robot[0]->origin = dummy->origin;
			cvResize( temp, robot[0] );
			temp = cvLoadImage( "resources/robot2.png" );
			robot[1] = cvCreateImage( cvSize( 300, 300 ), IPL_DEPTH_8U, 3 );
			robot[1]->origin = dummy->origin;
			cvResize( temp, robot[1] );
			temp = cvLoadImage( "resources/robot3.png" );
			robot[2] = cvCreateImage( cvSize( 300, 300 ), IPL_DEPTH_8U, 3 );
			robot[2]->origin = dummy->origin;
			cvResize( temp, robot[2] );

			temp = cvLoadImage( "resources/stone_icon.png" );
			icons[0] = cvCreateImage( cvSize( 60, 60 ), IPL_DEPTH_8U, 3 );
			icons[0]->origin = dummy->origin;
			cvResize( temp, icons[0] );
			temp = cvLoadImage( "resources/paper_icon.png" );
			icons[1] = cvCreateImage( cvSize( 60, 60 ), IPL_DEPTH_8U, 3 );
			icons[1]->origin = dummy->origin;
			cvResize( temp, icons[1] );
			temp = cvLoadImage( "resources/scissors_icon.png" );
			icons[2] = cvCreateImage( cvSize( 60, 60 ), IPL_DEPTH_8U, 3 );
			icons[2]->origin = dummy->origin;
			cvResize( temp, icons[2] );
			
			hr1->train( "hu_train3.txt" );
			if ( playerNum == 2 )
				hr2->train( "hu_train3.txt" );
		}
		
		
		void destroy( void ) {
			cvDestroyWindow( "Game" );
			cvDestroyWindow( "Seg" );
			if ( WINDOW_ARM1 ) cvDestroyWindow( "Arm1" );
			if ( WINDOW_HAND1 ) cvDestroyWindow( "Hand1" );
			if ( playerNum == 2 ) {
				if ( WINDOW_ARM2 ) cvDestroyWindow( "Arm2" );
				if ( WINDOW_HAND2 ) cvDestroyWindow( "Hand2" );
			}
		}
		
		bool finished( void ){
			int S = posList[0].size( );
			DEBUG( "f1" );
			int sum = 0;
			for( int i = 1; i <= 7; i++ ) {
				sum += abs( posList[0][S-i]-posList[0][S-i-1] );
			}
			DEBUG( "f2" );
			sum /= 7;
			DEBUG( "f3" );
			
			if (sum < 5 ) return true;
			DEBUG( "f3" );

			return false;
		}
		
		void finish( void ) {
			cout << "Finished!"<< endl;


			IplImage * temp = cvCreateImage( cvSize( frame->width, frame->height ), IPL_DEPTH_8U, 3 );
			
			int sign1, sign2;

			DEBUG( "f1" );
			cout << "f1";
			
			if( playerNum == 2 ) {
				RSPtype type1 = hr1->classify( hand1 );
				RSPtype hist1 = hr1->history( type1 );
				
				RSPtype type2 = hr2->classify( hand2 );
				RSPtype hist2 = hr2->history( type2 );
				
				switch( hist1 ) {
					case rspRock:
						sign1 = STONE;
						break;
					case rspPaper: case rspPaper2:
						sign1 = PAPER;
						break;
					case rspScissors:
						sign1 = SCISS;
						break;
					default: sign1 = STONE;				
				}
				switch( hist2 ) {
					case rspRock:
						sign2 = STONE;
						break;
					case rspPaper: case rspPaper2:
						sign2 = PAPER;
						break;
					case rspScissors:
						sign2 = SCISS;
						break;
					default: sign2 = STONE;				
				}
				
			} else {
				DEBUG( "fa2" );
				sign1 = rand( ) % 3;

				cout << "f2";
				RSPtype type1 = hr1->classify( hand1);
				RSPtype hist1 = type1;//hr1->history( type1 );
				cout << "f3";
				DEBUG( "fa3" );
				switch( hist1 ) {
					case rspRock:
						sign2 = STONE;
						break;
					case rspPaper: case rspPaper2:
						sign2 = PAPER;
						break;
					case rspScissors:
						sign2 = SCISS;
						break;
					default: sign2 = STONE;				
				}
				DEBUG( "fa4" );
			}
			
			int winner;
			
			if ( sign1 == sign2 ) winner = 0;
			else {
				if ( sign1==PAPER && sign2==STONE ) winner = 1;
				if ( sign2==PAPER && sign1==STONE ) winner = 2;

				if ( sign1==PAPER && sign2==SCISS ) winner = 2;
				if ( sign2==PAPER && sign1==SCISS ) winner = 1;

				if ( sign1==STONE && sign2==SCISS ) winner = 1;
				if ( sign2==STONE && sign1==SCISS ) winner = 2;
			}
			
			int signNum;
			
			if ( winner == 0 )
				signNum = 8;
			else
				if ( playerNum == 1 ) {
					if ( winner == 1 )
						signNum = 6;
					else
						signNum = 7;
				} else {
					if ( winner == 1 )
						signNum = 4;
					else
						signNum = 5;
				}
			
			cvCopy( frame, temp );
			temp->origin = frame->origin;
			
			for( float i = 0.0f; i <= 1.0f; i+= 0.04f ) {
				if ( playerNum == 1 )
					drawRobot( sign1 );
					
				drawIcon( temp, sign1, 0 );
				drawIcon( temp, sign2, 1 );
				drawSign( temp, signNum, i );
				
				cvShowImage( "Game", temp );
				cvCopy( frame, temp );
				
				if ( ( cvWaitKey( 200 ) & 255 ) == 27 )
					break;
			}
			while( 1 )
				if ( ( cvWaitKey( 20 ) & 255 ) == 27 )
					break;
		}
		
		void drawRobot( int N ) {
			int midx = 0;
			int midy = -20;
			int x, y;
			unsigned char r, g, b;
			
			for( int i = posList[0].size( ) - 1; i >= 0; i-- )
				if ( posList[0][i] != 0 ) {
					midx = posList[0][i];
					break;
				}
					
			IplImage * rob = robot[N];
			
			for( int i = 0; i < rob->height; i++ )
				for( int j = 0; j < rob->width; j++ ) {
					x = midx + i - rob->height / 2;
					y = midy + j - rob->width / 2;
					
					if ( x < 0 || y < 0 || x > frame->height-1 || y > frame->width-1 )
						continue;
						
					r = rob->imageData[ i*rob->widthStep+j*3 + 0];
					g = rob->imageData[ i*rob->widthStep+j*3 + 1];
					b = rob->imageData[ i*rob->widthStep+j*3 + 2];
					
					if ( r==0 && g==0 && b == 0 )
						continue;
					
					frame->imageData[ x*frame->widthStep+y*3 +0 ] = rob->imageData[ i*rob->widthStep+j*3 + 0];
					frame->imageData[ x*frame->widthStep+y*3 +1 ] = rob->imageData[ i*rob->widthStep+j*3 + 1];
					frame->imageData[ x*frame->widthStep+y*3 +2 ] = rob->imageData[ i*rob->widthStep+j*3 + 2];
				}
		}
		
		void run( void ) {
			init( );
		        while( true ) {
				frame = capture->getFrame( ); 
				
				DEBUG( "1" );
				
				if ( !frame ) break;
				
				DEBUG( "2" );
			
				segmentation->segmentate( frame, seg );
				cvShowImage( "Seg", seg );
				
				if ( playerNum == 2 )
					ArmExtractor::extract( seg, arm1, arm2 );
				else
					ArmExtractor::extract( seg, arm1 );

				DEBUG( "3" );
				
					
				if ( WINDOW_ARM1 ) cvShowImage( "Arm1", arm1 );
				cvCopy( arm1, hand1 ); hand1->origin = arm1->origin;

				DEBUG( "3a" );
				if ( playerNum == 2 )
					cvFlip( hand1, NULL, 1);
				recognizer.cutHand( hand1 );
				
				RSPtype type1 = hr1->classify( hand1 );
				RSPtype hist1 = hr1->history( type1 );
				cout << handType(type1) << "    " << handType(hist1) << "         ";
				if ( playerNum == 1 ) cout << endl;
				
				if ( playerNum == 2 ) {
					RSPtype type2 = hr2->classify( hand2 );
					RSPtype hist2 = hr2->history( type2 );
					cout << handType(type2) << "    " << handType(hist2) << endl;
				}
				
				
				
				
				DEBUG( "3b" );
				if ( WINDOW_HAND1 ) cvShowImage( "Hand1", hand1 );

				DEBUG( "3c" );

				if ( playerNum == 2 ) {
					if ( WINDOW_ARM2 ) cvShowImage( "Arm2", arm2);
					cvCopy( arm2, hand2 ); hand2->origin = arm2->origin;
					recognizer.cutHand( hand2 );
					if ( WINDOW_HAND2 ) cvShowImage( "Hand2", hand2 );
				}
						   
				storePositions( );
				DEBUG( "3d" );
				
				if ( !started ) {
					cvShowImage( "Game", frame );
					started = detectStarted( );
					
					if ( started )
						cout << "Started!!!" << endl;
				}
				
				DEBUG( "4" );
				
				
				if ( cdnum == 3 && finished() ) {
					finish( );
					break;
				}
				
				if ( started && playerNum == 1 ) {
					drawRobot( 0 );
				}

				DEBUG( "5" );
				
				if ( started )	
				switch ( getMovement( ) ) {
					case  0:
						cout << "-------" << endl; 
						break;
					case -1:
						if ( direction == 1 ) {
							direction = -1;
							if ( cdnum < 3 ) {
								cdnum++;
								
								alphastate = 0;
							}
						}
						cout << "down!!!" << endl;
						break;
					case  1:
						cout << "up!!!!!" << endl;
						if ( direction == -1 ) {
							direction = 1;
						}
						break;
				}
				
				if ( alphastate != -1 && alphastate < 1	 ) {
					if ( alphastate < 1 )
						alphastate += 0.1f;
						drawSign( frame, cdnum, alphastate );
				}
				
				cvShowImage( "Game", frame );

				DEBUG( "6" );
				
				if ( ( cvWaitKey( 10 ) & 255 ) == 27 )
					break;
		        }
		}
		
};

int main( int argc, char ** argv ) {
	VideoSource * capture;
	int playerNum;

	if ( argc < 2 || argc > 3) {
		cout << "Usage: programname playernum [videofilename]" << endl;
		return -1;
	}
	if ( sscanf( argv[1], "%d", &playerNum ) != 1 ) {
		cout << "Usage: programname playernum [videofilename]" << endl;
		return -1;
	}
	

	if ( argc == 2 ) {
		capture = new VideoSource( 0 );
	}
	if ( argc == 3 ) {
		capture = new VideoSource( argv[2] );
	}
	
	Game * game = new Game( capture, playerNum );
	
	game->run( );


}

